@@ -14,8 +14,14 @@
 #include "change-db-cb.h"
 #include "new-db-cb.h"
 #include "common/common.h"
-#include "version.h"
 #include "secret-schema.h"
+#include "change-pwd-cb.h"
+#include "settings-cb.h"
+#include "shortcuts-cb.h"
+#include "webcam-add-cb.h"
+#include "manual-add-cb.h"
+#include "edit-row-cb.h"
+#include "show-qr-cb.h"
 
 
 #ifndef USE_FLATPAK_APP_FOLDER
@@ -52,6 +58,8 @@ static void       get_window_size_cb        (GtkWidget          *window,
                                              GtkAllocation      *allocation,
                                              gpointer            user_data);
 
+void              setup_kb_shortcuts        (AppData            *app_data);
+
 static void       toggle_button_cb          (GtkWidget          *main_window,
                                              gpointer            user_data);
 
@@ -61,10 +69,6 @@ static void       reorder_rows_cb           (GtkToggleButton *btn,
 static void       del_data_cb               (GtkToggleButton    *btn,
                                              gpointer            user_data);
 
-static void       change_password_cb        (GSimpleAction      *simple,
-                                             GVariant           *parameter,
-                                             gpointer            user_data);
-
 static void       save_sort_order           (GtkTreeView        *tree_view);
 
 static void       save_window_size          (gint                width,
@@ -246,6 +250,7 @@ activate (GtkApplication    *app,
     app_data->clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
 
     create_treeview (app_data);
+    setup_kb_shortcuts (app_data);
 
     app_data->notification = g_notification_new ("OTPClient");
     g_notification_set_priority (app_data->notification, G_NOTIFICATION_PRIORITY_NORMAL);
@@ -254,18 +259,12 @@ activate (GtkApplication    *app,
     g_notification_set_body (app_data->notification, "OTP value has been copied to the clipboard");
     g_object_unref (icon);
 
-    GtkBindingSet *binding_set = gtk_binding_set_by_class (GTK_APPLICATION_WINDOW_GET_CLASS (app_data->main_window));
-
     GtkToggleButton *reorder_toggle_btn = GTK_TOGGLE_BUTTON(gtk_builder_get_object (app_data->builder, "reorder_toggle_btn_id"));
-    g_signal_new ("toggle-reorder-button", G_TYPE_OBJECT, G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION, 0, NULL, NULL, NULL, G_TYPE_NONE, 0);
-    gtk_binding_entry_add_signal (binding_set, GDK_KEY_r, GDK_CONTROL_MASK, "toggle-reorder-button", 0);
     g_signal_connect (app_data->main_window, "toggle-reorder-button", G_CALLBACK(toggle_button_cb), reorder_toggle_btn);
     g_signal_connect (reorder_toggle_btn, "toggled", G_CALLBACK(reorder_rows_cb), app_data);
     g_signal_connect (app_data->main_window, "key_press_event", G_CALLBACK(key_pressed_cb), NULL);
 
     GtkToggleButton *del_toggle_btn = GTK_TOGGLE_BUTTON(gtk_builder_get_object (app_data->builder, "del_toggle_btn_id"));
-    g_signal_new ("toggle-delete-button", G_TYPE_OBJECT, G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION, 0, NULL, NULL, NULL, G_TYPE_NONE, 0);
-    gtk_binding_entry_add_signal (binding_set, GDK_KEY_d, GDK_CONTROL_MASK, "toggle-delete-button", 0);
     g_signal_connect (app_data->main_window, "toggle-delete-button", G_CALLBACK(toggle_button_cb), del_toggle_btn);
     g_signal_connect (del_toggle_btn, "toggled", G_CALLBACK(del_data_cb), app_data);
     g_signal_connect (app_data->main_window, "key_press_event", G_CALLBACK(key_pressed_cb), NULL);
@@ -341,7 +340,7 @@ key_pressed_cb (GtkWidget   *window,
 
 
 static GKeyFile *
-get_kf_ptr ()
+get_kf_ptr (void)
 {
     GError *err = NULL;
     GKeyFile *kf = g_key_file_new ();
@@ -357,6 +356,7 @@ get_kf_ptr ()
             return kf;
         }
         g_printerr ("%s\n", err->message);
+        g_clear_error (&err);
     }
     g_free (cfg_file_path);
     g_key_file_free (kf);
@@ -387,7 +387,7 @@ get_wh_data (gint     *width,
 
 
 static gboolean
-get_warn_data ()
+get_warn_data (void)
 {
     GKeyFile *kf = get_kf_ptr ();
     gboolean show_warning = TRUE;
@@ -420,6 +420,7 @@ set_warn_data (gboolean show_warning)
 #endif
         if (!g_key_file_save_to_file (kf, cfg_file_path, &err)) {
             g_printerr ("%s\n", err->message);
+            g_clear_error (&err);
         }
         g_free (cfg_file_path);
         g_key_file_free (kf);
@@ -437,9 +438,6 @@ create_main_window (gint             width,
 
     gtk_window_set_default_size (GTK_WINDOW(app_data->main_window), (width >= 150) ? width : 500, (height >= 150) ? height : 300);
 
-    GtkWidget *header_bar =  GTK_WIDGET(gtk_builder_get_object (app_data->builder, "headerbar_id"));
-    gtk_header_bar_set_subtitle (GTK_HEADER_BAR(header_bar), PROJECT_VER);
-
     GtkWidget *lock_btn = GTK_WIDGET(gtk_builder_get_object (app_data->builder, "lock_btn_id"));
     g_signal_connect (lock_btn, "clicked", G_CALLBACK(lock_app), app_data);
 
@@ -448,7 +446,7 @@ create_main_window (gint             width,
 
 
 static gboolean
-show_upgrade_msg ()
+show_upgrade_msg (void)
 {
     gboolean show_msg = TRUE;
     GKeyFile *kf = get_kf_ptr ();
@@ -496,9 +494,9 @@ on_bar_response (GtkInfoBar *ib,
 #else
         cfg_file_path = g_build_filename (g_get_user_data_dir (), "otpclient.cfg", NULL);
 #endif
-        g_key_file_save_to_file (kf, cfg_file_path, &err);
-        if (err != NULL) {
+        if (!g_key_file_save_to_file (kf, cfg_file_path, &err)) {
             g_printerr ("%s\n", err->message);
+            g_clear_error (&err);
         }
         g_free (cfg_file_path);
     }
@@ -516,7 +514,6 @@ set_action_group (GtkBuilder *builder,
     static GActionEntry settings_menu_entries[] = {
             { .name = ANDOTP_IMPORT_ACTION_NAME, .activate = select_file_cb },
             { .name = ANDOTP_IMPORT_PLAIN_ACTION_NAME, .activate = select_file_cb },
-            { .name = AUTHPLUS_IMPORT_ACTION_NAME, .activate = select_file_cb },
             { .name = FREEOTPPLUS_IMPORT_ACTION_NAME, .activate = select_file_cb },
             { .name = AEGIS_IMPORT_ACTION_NAME, .activate = select_file_cb },
             { .name = AEGIS_IMPORT_ENC_ACTION_NAME, .activate = select_file_cb },
@@ -526,20 +523,22 @@ set_action_group (GtkBuilder *builder,
             { .name = AEGIS_EXPORT_ACTION_NAME, .activate = export_data_cb },
             { .name = AEGIS_EXPORT_PLAIN_ACTION_NAME, .activate = export_data_cb },
             { .name = GOOGLE_MIGRATION_FILE_ACTION_NAME, .activate = add_qr_from_file },
-            { .name = GOOGLE_MIGRATION_WEBCAM_ACTION_NAME, .activate = webcam_cb },
+            { .name = GOOGLE_MIGRATION_WEBCAM_ACTION_NAME, .activate = webcam_add_cb },
             { .name = "create_newdb", .activate = new_db_cb },
             { .name = "change_db", .activate = change_db_cb },
             { .name = "change_pwd", .activate = change_password_cb },
-            { .name = "edit_row", .activate = edit_selected_row_cb },
+            { .name = "edit_row", .activate = edit_row_cb },
+            { .name = "show_qr", .activate = show_qr_cb },
             { .name = "settings", .activate = settings_dialog_cb },
-            { .name = "shortcuts", .activate = shortcuts_window_cb }
+            { .name = "shortcuts", .activate = shortcuts_window_cb },
+            { .name = "about", .activate = about_diag_cb }
     };
 
     static GActionEntry add_menu_entries[] = {
-            { .name = "webcam", .activate = webcam_cb },
+            { .name = "webcam", .activate = webcam_add_cb },
             { .name = "import_qr_file", .activate = add_qr_from_file },
             { .name = "import_qr_clipboard", .activate = add_qr_from_clipboard },
-            { .name = "manual", .activate = add_data_dialog }
+            { .name = "manual", .activate = manual_add_cb }
     };
 
     GtkWidget *settings_popover = GTK_WIDGET (gtk_builder_get_object (builder, "settings_pop_id"));
@@ -571,9 +570,10 @@ get_db_path (AppData *app_data)
         if (!g_key_file_load_from_file (kf, cfg_file_path, G_KEY_FILE_NONE, &err)) {
             show_message_dialog (app_data->main_window, err->message, GTK_MESSAGE_ERROR);
             g_key_file_free (kf);
+            g_clear_error (&err);
             return NULL;
         }
-        db_path = g_key_file_get_string (kf, "config", "db_path", &err);
+        db_path = g_key_file_get_string (kf, "config", "db_path", NULL);
         if (db_path == NULL) {
             goto new_db;
         }
@@ -604,9 +604,9 @@ get_db_path (AppData *app_data)
     if (res == GTK_RESPONSE_ACCEPT) {
         db_path = gtk_file_chooser_get_filename (chooser);
         g_key_file_set_string (kf, "config", "db_path", db_path);
-        g_key_file_save_to_file (kf, cfg_file_path, &err);
-        if (err != NULL) {
+        if (!g_key_file_save_to_file (kf, cfg_file_path, &err)) {
             g_printerr ("%s\n", err->message);
+            g_clear_error (&err);
         }
     }
 
@@ -682,32 +682,6 @@ del_data_cb (GtkToggleButton *btn,
 }
 
 
-static void
-change_password_cb (GSimpleAction *simple    __attribute__((unused)),
-                    GVariant      *parameter __attribute__((unused)),
-                    gpointer       user_data)
-{
-    AppData *app_data = (AppData *)user_data;
-    gchar *tmp_key = secure_strdup (app_data->db_data->key);
-    gchar *pwd = prompt_for_password (app_data, tmp_key, NULL, FALSE);
-    if (pwd != NULL) {
-        app_data->db_data->key = pwd;
-        GError *err = NULL;
-        update_and_reload_db (app_data, app_data->db_data, FALSE, &err);
-        if (err != NULL) {
-            show_message_dialog (app_data->main_window, err->message, GTK_MESSAGE_ERROR);
-            GtkApplication *app = gtk_window_get_application (GTK_WINDOW(app_data->main_window));
-            destroy_cb (app_data->main_window, app_data);
-            g_application_quit (G_APPLICATION(app));
-        }
-        show_message_dialog (app_data->main_window, "Password successfully changed", GTK_MESSAGE_INFO);
-        secret_password_store (OTPCLIENT_SCHEMA, SECRET_COLLECTION_DEFAULT, "main_pwd", app_data->db_data->key, NULL, on_password_stored, NULL, "string", "main_pwd", NULL);
-    } else {
-        gcry_free (tmp_key);
-    }
-}
-
-
 static void
 get_window_size_cb (GtkWidget      *window,
                     GtkAllocation  *allocation __attribute__((unused)),
@@ -725,9 +699,6 @@ destroy_cb (GtkWidget   *window,
             gpointer     user_data)
 {
     AppData *app_data = (AppData *)user_data;
-    if (app_data->disable_secret_service == TRUE) {
-        secret_password_clear (OTPCLIENT_SCHEMA, NULL, on_password_cleared, NULL, "string", "main_pwd", NULL, NULL);
-    }
     save_sort_order (app_data->tree_view);
     g_source_remove (app_data->source_id);
     g_source_remove (app_data->source_id_last_activity);
@@ -793,11 +764,13 @@ store_data (const gchar *param1_name,
     if (g_file_test (cfg_file_path, G_FILE_TEST_EXISTS)) {
         if (!g_key_file_load_from_file (kf, cfg_file_path, G_KEY_FILE_NONE, &err)) {
             g_printerr ("%s\n", err->message);
+            g_clear_error (&err);
         } else {
             g_key_file_set_integer (kf, "config", param1_name, param1_value);
             g_key_file_set_integer (kf, "config", param2_name, param2_value);
             if (!g_key_file_save_to_file (kf, cfg_file_path, &err)) {
                 g_printerr ("%s\n", err->message);
+                g_clear_error (&err);
             }
         }
     }
